﻿using System.Text;
using System.Net.Http.Headers;

using Newtonsoft.Json;

namespace TestProject;

/// <summary>
/// API 클라이언트
/// </summary>
public class APIClient : IDisposable
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// 리소스 해제 여부
    /// </summary>
    private bool disposed = false;

    /// <summary>
    /// BASE URL
    /// </summary>
    private const string BASE_URL = "https://clovastudio.stream.ntruss.com/testapp/v1/chat-completions/HCX-DASH-001";

    /// <summary>
    /// 클로바 스튜디오 API 키
    /// </summary>
    private string clovaStudioAPIKey;

    /// <summary>
    /// API 게이트웨이 API 키
    /// </summary>
    private string apiGatewayAPIKey;

    /// <summary>
    /// HTTP 클라이언트
    /// </summary>
    private HttpClient client;

    /// <summary>
    /// 품질을 갖는 스트림 미디어 타입 헤더 값
    /// </summary>
    private MediaTypeWithQualityHeaderValue streamMediaTypeWithQualityHeaderValue;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Property
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 클로바 스튜디오 API 키 - ClovaStudioAPIKey

    /// <summary>
    /// 클로바 스튜디오 API 키
    /// </summary>
    public string ClovaStudioAPIKey
    {
        get
        {
            return this.clovaStudioAPIKey;
        }
    }

    #endregion
    #region API 게이트웨이 API 키 - APIGatewayAPIKey

    /// <summary>
    /// API 게이트웨이 API 키
    /// </summary>
    public string APIGatewayAPIKey
    {
        get
        {
            return this.apiGatewayAPIKey;
        }
    }

    #endregion
    #region HTTP 클라이언트 - Client

    /// <summary>
    /// HTTP 클라이언트
    /// </summary>
    public HttpClient Client
    {
        get
        {
            return this.client;
        }
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region API 클라이언트 - APIClient(clovaStudioAPIKey, apiGatewayAPIKey)

    /// <summary>
    /// API 클라이언트
    /// </summary>
    /// <param name="clovaStudioAPIKey">클로바 스튜디오 API 키</param>
    /// <param name="apiGatewayAPIKey">API 게이트웨이 API 키</param>
    public APIClient(string clovaStudioAPIKey, string apiGatewayAPIKey)
    {
        this.clovaStudioAPIKey = clovaStudioAPIKey;
        this.apiGatewayAPIKey  = apiGatewayAPIKey;

        this.client = new HttpClient();

        this.client.DefaultRequestHeaders.Add("X-NCP-CLOVASTUDIO-API-KEY", this.clovaStudioAPIKey);
        this.client.DefaultRequestHeaders.Add("X-NCP-APIGW-API-KEY"      , this.apiGatewayAPIKey );

        this.streamMediaTypeWithQualityHeaderValue = new MediaTypeWithQualityHeaderValue("text/event-stream");
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 채팅 요청하기 - RequestChat(requestMessage)

    /// <summary>
    /// 채팅 요청하기
    /// </summary>
    /// <param name="requestMessage">요청 메시지</param>
    /// <returns>응답 메시지 태스크</returns>
    public async Task<ResponseMessage> RequestChat(RequestMessage requestMessage)
    {
        if(this.client.DefaultRequestHeaders.Contains("X-NCP-CLOVASTUDIO-REQUEST-ID"))
        {
            this.client.DefaultRequestHeaders.Remove("X-NCP-CLOVASTUDIO-REQUEST-ID");
        }

        string clovaStudioRequestID = Guid.NewGuid().ToString();

        this.client.DefaultRequestHeaders.Add("X-NCP-CLOVASTUDIO-REQUEST-ID", clovaStudioRequestID);

        string requestJSON = JsonConvert.SerializeObject(requestMessage);

        StringContent content = new StringContent(requestJSON, Encoding.UTF8, "application/json");

        HttpResponseMessage responseMessage = await this.client.PostAsync(BASE_URL, content);

        responseMessage.EnsureSuccessStatusCode();

        string responseString = await responseMessage.Content.ReadAsStringAsync();

        return JsonConvert.DeserializeObject<ResponseMessage>(responseString);
    }

    #endregion
    #region 리소스 해제하기 - Dispose()

    /// <summary>
    /// 리소스 해제하기
    /// </summary>
    public void Dispose()
    {
        Dispose(true);

        GC.SuppressFinalize(this);
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Protected

    #region 리소스 해제하기 - Dispose(disposing)

    /// <summary>
    /// 리소스 해제하기
    /// </summary>
    /// <param name="disposing">리소스 해제 여부</param>
    protected virtual void Dispose(bool disposing)
    {
        if(!this.disposed)
        {
            if(disposing)
            {
                if(this.client != null)
                {
                    this.client.Dispose();
                }

                this.client = null;

                this.disposed = true;
            }
        }
    }

    #endregion
}